#include "ac_plugin.h"
#include "Undoables.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

//---------------------------------------------------------------------------------------
static void ddhseltex_selObjsWithTex( List **l, ACObject *o, int idx )
{
   for( List *p = ac_object_get_childrenlist( o ); p != NULL; p = p->next )
   {
      ACObject *obj = ( ACObject * )p->data;
      if( ac_object_get_type( obj ) == OBJECT_NORMAL )
      {
         if( !ac_object_has_texture( obj ))
         {
            if( idx < 0 )
            {
               list_add_item_head( l, obj );
            }
         }
         else if( ac_object_get_texture_index( obj ) == idx )
         {
            list_add_item_head( l, obj );
         }
      }
      ddhseltex_selObjsWithTex( l, obj, idx );
   }
}
//---------------------------------------------------------------------------------------
// Find an invalid texture within a group
static bool ddhFindException( ACObject *o, int idx )
{
   for( List *p = ac_object_get_childrenlist( o ); p != NULL; p = p->next )
   {
      ACObject *obj = ( ACObject * )p->data;
      if( ac_object_get_type( obj ) == OBJECT_NORMAL )
      {
         if( !ac_object_has_texture( obj ))
         {
            if( idx > -1 )
            {
               return true;
            }
         }
         else if( ac_object_get_texture_index( obj ) != idx )
         {
            return true;
         }
      }
      if( ddhFindException( obj, idx ))
      {
         return true;
      }
   }
   return false;
}
//---------------------------------------------------------------------------------------
// Select all textures with the given name
static void ddhseltex( int argc, char *argv[] )
{
   if( argc < 2 )
   {
      display_message( "seltex : no texture specified" );
      return;
   }
   add_undoable_change_selection( "select objects with texture" );
   clear_selection();
   List *l = NULL;
   int idx = atoi( argv[1] ) - 1;
   ddhseltex_selObjsWithTex( &l, ac_get_world(), idx );
   ac_selection_select_objectlist( l );
   int numsel = list_count( l );
   list_free( &l );
   // If we're in Group mode, check whether objects that have been selected
   // are all of the same texture.
   if( ac_get_select_mode() == SELECT_GROUP )
   {
      l = ac_selection_get_objects();
      bool foundExceptions = false;
      for( List *p = l; p != NULL; p = p->next )
      {
         ACObject *obj = ( ACObject * )p->data;
         if( ac_object_get_type( obj ) == OBJECT_GROUP )
         {
            if( ddhFindException( obj, idx ))
            {
               foundExceptions = true;
               break;
            }
         }
      }
      list_free( &l );
      if( foundExceptions )
      {
         message_dialog( "seltex : some objects with a different texture than the one specified were selected, because they belong to a group containing other textures.  To ensure you only select the objects with the requested texture, switch to group mode and issue the selection again," );
      }
   }
   redraw_all();
   display_status();
   display_message( "seltex : selected %d objects", numsel );
}
//---------------------------------------------------------------------------------------
// Select all textures with the given name
static void ddhseltex_showwin()
{

   tcl_command( "ddhseltex_dialog" );

   tcl_command( ".ddhseltex.frmlist.listtex insert end \"no texture\"" );
   int numtex = ac_get_num_textures();
   char *ch = NULL;
   for( int ix = 0; ix < numtex; ++ix )
   {
      ACImage *i = texture_id_to_image( ix );
      ac_entity_get_string_value(( ACEntity * )i, "name", &ch );
      tcl_command( ".ddhseltex.frmlist.listtex insert end \"%s\"", ch );
   }

}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC int AC3DPluginInit( AC3DPluginInitData *d )
{

   static bool firstTime = true;
   if( firstTime )
   {

      char *path = ( char * )strdup( d->plugin_dir_path );
#ifdef WINDOWS
      ac_unbackslash_string( path );
#endif

      tcl_command( "source \"%s/seltex.tcl\"", path );

      // Add textures to TCL list
//tcl_command( "set ddhknife_knife_name \"Unnamed\"" );

      ac_add_command( "ddhseltex_showwin", ddhseltex_showwin );
      ac_add_tools_menu_item( "Select objects with texture...", "ac3d ddhseltex_showwin",
                              "Select all objects with a given texture" );
      ac_add_command_full( "ddhseltex_go", ( void * )ddhseltex,
                            1, "argv", "<texture_index>",
                           "Select objects with texture" );

      firstTime = false;

   }

   return 0;

}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC int AC3DPluginExit()
{
   return 0;
}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC char *AC3DPluginAbout()
{
   return( "Select Objects With Texture - version 1.0.1 - Dennis Hawthorne" );
}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC char *AC3DPluginInfo()
{
   return( "Select Objects With Texture utility - Selects all objects which have been mapped with a given texture" );
}
//---------------------------------------------------------------------------------------

